/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Main win32 application
 */

#include "anya.h"
#include "oz.h"

#include <stdlib.h>

extern LRESULT CALLBACK AnyaWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

static void
anya_window_register_class(HINSTANCE hInstance)
{
    WNDCLASSW wc;
    wc.style            = CS_DBLCLKS;
    wc.lpfnWndProc      = AnyaWndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = hInstance;
    wc.hIcon            = 
#ifndef _WIN32_WCE
                          LoadIconW(hInstance, MAKEINTRESOURCEW(IDI_APPLICATION));
#else
                          NULL;
#endif
    wc.hCursor          = NULL;
#ifndef _DEBUG
    wc.hbrBackground    = GetStockObject(WHITE_BRUSH);
#else
    /* set the background to black in debug builds so that we can better 
       see the areas not being painted */
    wc.hbrBackground    = GetStockObject(BLACK_BRUSH);
#endif
    wc.lpszMenuName     = NULL;
    wc.lpszClassName    = L"AnyaWndClass";

    RegisterClassW(&wc);
}

static int
already_running()
{
    HWND hwnd = FindWindow(L"AnyaWndClass", NULL);
    if (hwnd) {
        /* set focus to foremost child window
           The "| 0x01" is used to bring any owned windows to the foreground and
           activate them. */
        SetForegroundWindow((HWND)((ULONG)hwnd | 0x00000001));
        return 1;
    }
    return 0;
}

#ifdef _WIN32_WCE
#define wWinMain WinMain
#endif

int __stdcall
wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nShowCmd)
{
    AnyaWindow *win;
    MSG msg;
    int menuheight = 0; /* for WinCE */

    if (already_running())
        return FALSE;

    anya_window_register_class(hInstance);

    oz_events_init();

#ifdef _WIN32_WCE
    menuheight = GetSystemMetrics(SM_CYMENU);
#endif

    win = anya_window_create(0, menuheight, oz_screen_get_width(), oz_screen_get_height() - menuheight);
    oz_window_show(win->window);

    anya_window_load(win, CLIENT_START_PAGE, ANYA_REQUEST_GET);

    while (GetMessage(&msg, NULL,  0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    anya_window_destroy(win);

    oz_events_shutdown();

    return msg.wParam;
}
