/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 1998 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Roy Yokoyama <yokoyama@netscape.com>
 *    Simon Montagu <smontagu@netscape.com>
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 */

/* Based on CVS revision 1.6 of mozilla/intl/uconv/src/nsUTF8ToUnicode.cpp
 * http://bonsai.mozilla.org/cvsblame.cgi?file=mozilla/intl/uconv/src/nsUTF8ToUnicode.cpp
 */

/* XXX rev 1.7 is available now with some bug fixes.  update to that */

int
utf8_to_ucs2(const char *aSrc,      unsigned long *aSrcLength,
             unsigned short *aDest, unsigned long *aDestLength)
{
    unsigned long mState = 0;   // cached expected number of bytes per UTF8 character sequence
    unsigned long mUcs4  = 0;   // cached Unicode character
    unsigned long mBytes = 1;

    unsigned long aSrcLen  = *aSrcLength;
    unsigned long aDestLen = *aDestLength;

    const char *in, *inend;
    unsigned short *out, *outend;

    int res;    // conversion result

    inend = aSrc + aSrcLen;
    outend = aDest + aDestLen;

    for(in=aSrc,out=aDest,res=1;((in < inend) && (out < outend)); in++)
    {
        if(0 == mState) {
            if( 0 == (0x80 & (*in))) {
                // ASCII
                *out++ = (unsigned short)*in;
                mBytes =1;
            } else if( 0xC0 == (0xE0 & (*in))) {
                // 2 bytes UTF8
                mUcs4 = (unsigned long)(*in);
                mUcs4 = (mUcs4 << 6) & 0x000007C0L;
                mState=1;
                mBytes =2;
            } else if( 0xE0 == (0xF0 & (*in))) {
                // 3 bytes UTF8
                mUcs4 = (unsigned long)(*in);
                mUcs4 = (mUcs4 << 12) & 0x0000F000L;
                mState=2;
                mBytes =3;
            } else if( 0xF0 == (0xF8 & (*in))) {
                // 4 bytes UTF8
                mUcs4 = (unsigned long)(*in);
                mUcs4 = (mUcs4 << 18) & 0x001F0000L;
                mState=3;
                mBytes =4;
            } else if( 0xF8 == (0xFC & (*in))) {
                // 5 bytes UTF8
                mUcs4 = (unsigned long)(*in);
                mUcs4 = (mUcs4 << 24) & 0x03000000L;
                mState=4;
                mBytes =5;
            } else if( 0xFC == (0xFE & (*in))) {
                // 6 bytes UTF8
                mUcs4 = (unsigned long)(*in);
                mUcs4 = (mUcs4 << 30) & 0x40000000L;
                mState=5;
                mBytes =6;
            } else {
                // NS_ASSERTION(0, "The input string is not in utf8");
                // unexpected octet, put in a replacement char, 
                // flush and refill the buffer, reset state
                res = 0;
                break;
            }
        } else {
            if (0x80 == (0xC0 & (*in))) {
                unsigned long tmp = (*in);
                unsigned long shift = (mState-1) * 6;
                tmp = (tmp << shift ) & ( 0x0000003FL << shift);
                mUcs4 |= tmp;
                if (0 == --mState) {
                    if (mUcs4 >= 0x00010000) {
                        if (mUcs4 >= 0x00110000) {
                            *out++ = 0xFFFD;
                        } else {
                            mUcs4 -= 0x00010000;
                            *out++ = 0xD800 | (0x000003FF & (mUcs4 >> 10));
                            *out++ = 0xDC00 | (0x000003FF & mUcs4);
                        }
                    } else {
                        // from Unicode 3.1, non-shortest form is illegal 
                        if(((2==mBytes) && (mUcs4 < 0x0080)) ||
                           ((3==mBytes) && (mUcs4 < 0x0800)) ||
                           ((4==mBytes) && (mUcs4 < 0x1000)) ||
                            (5==mBytes) ||
                            (6==mBytes)) 
                        {
                            res = 0;
                            break;
                        } 

                        if (0xfeff != mUcs4) // ignore BOM
                        {  
                            *out++ = mUcs4;
                        }
                    }
                    //initialize UTF8 cache
                    mState = mUcs4 = 0;
                    mBytes = 1;
                }
            } else {
                // NS_ASSERTION(0, "The input string is not in utf8");
                // unexpected octet, put in a replacement char, 
                // flush and refill the buffer, reset state
                in--;
                res = 0;
                break;
            }
        }
    }

    // output not finished, output buffer too short
    if ((1 == res) && (in < inend) && (out >= outend)) 
        res = 2;

    // last USC4 is incomplete, make sure the caller 
    // returns with properly aligned continuation of the buffer
    if ((1 == res) && (mState != 0))
        res = 2;

    *aSrcLength = in - aSrc;
    *aDestLength = out - aDest;

    return (res);
}
