/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Arc/Circle/Ellipse drawing code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "oz.h"
#include "ozprivate.h"

#include <math.h>

#ifndef M_PI
#define M_PI 3.14159265358979323846f
#endif

#ifdef _WIN32
#define tanf(x) (float)tan((double)(x))
#endif

#ifdef _WIN32_WCE

struct ArcBuf {
    const ozWindow *win;
    const ozGC *gc;
    int n;
    const int xoff;
    const int yoff;
    const float start;
    const float stan;
    const float end;
    const float etan;
};

/* XXX Make this function not suck */
static void
EllipsePoints(struct ArcBuf *ab, int x, int y)
{
    int i = 0;
    int drawQuad1=0, drawQuad2=0, drawQuad3=0, drawQuad4=0;

    /* quad 1 */
    if (ab->start <= 90) {
        if (ab->start == 0 && ab->end >= 90) {
            drawQuad1 = TRUE;
        } else {
            if (x != 0) {
                const float yox = (float)y/(float)x;
                if (yox >= ab->stan)
                    if (ab->end > 90 || yox <= ab->etan)
                        drawQuad1 = TRUE;
            } else {
                if (ab->start == 90 || ab->end == 90)
                    drawQuad1 = TRUE;
            }
        }
    }

    /* quad 2 */
    if (ab->start <= 180 && ab->end > 90) {
        if (ab->start <= 91 && ab->end >= 180) {
            drawQuad2 = TRUE;
        } else {
            if (x != 0) {
                const float yox = (float)y/(float)-x;
                if (ab->start < 90 || yox >= ab->stan)
                    if (ab->end > 180 || yox <= ab->etan)
                        drawQuad2 = TRUE;
            } else {
                if (ab->start <= 90 || ab->end >= 90)
                    drawQuad2 = TRUE;
            }
        }
    }

    /* quad 3 */
    if (ab->start <= 270 && ab->end > 180) {
        if (ab->start <= 181 && ab->end >= 270) {
            drawQuad3 = TRUE;
        } else {
            if (x != 0) {
                const float yox = (float)-y/(float)-x;
                if (ab->start <= 180 || yox >= ab->stan)
                    if (ab->end >= 270 || yox <= ab->etan)
                        drawQuad3 = TRUE;
            } else {
                if (ab->start == 270 || ab->end == 270)
                    drawQuad3 = TRUE;
            }
        }
    }

    /* quad 4 */
    if (ab->start <= 360 && ab->end > 270) {
        if (ab->start <= 271 && ab->end >= 360) {
            drawQuad4 = TRUE;
        } else {
            if (x != 0) {
                const float yox = (float)-y/(float)x;

                if (ab->start < 270 || yox >= ab->stan)
                    if (yox <= ab->etan) /* XXX avoid drawing 0 twice */
                        drawQuad4 = TRUE;
            } else {
                if (ab->start <= 360 || ab->end >= 360)
                    drawQuad4 = TRUE;
            }
        }
    }

    if (drawQuad1) ++i;
    if (drawQuad2) ++i;
    if (drawQuad3) ++i;
    if (drawQuad4) ++i;

    if (i > 0) {
        if (drawQuad1) /* upper right */
            SetPixel(ab->gc->dc, (x + ab->xoff), (ab->yoff - y), ab->gc->rgbaq);

        if (drawQuad2) /* upper left */
            SetPixel(ab->gc->dc, (ab->xoff - x), (ab->yoff - y), ab->gc->rgbaq);

        if (drawQuad3) /* bottom left */
            SetPixel(ab->gc->dc, (ab->xoff - x), (y + ab->yoff), ab->gc->rgbaq);

        if (drawQuad4) /* bottom right */
            SetPixel(ab->gc->dc, (x + ab->xoff), (y + ab->yoff), ab->gc->rgbaq);

        ab->n += i;
    }
}

/* Based on code from
 * Computer Graphics: Principles and Practice
 * by Foley, van Dam, Feiner, Hughes
 */
static void
MidpointEllipse(struct ArcBuf *ab, int a, int b)
{
    const int a2 = a * a;
    const int b2 = b * b;

    int x = 0;
    int y = b;

    /* These were originally doubles, but floats are much faster */
    float d1 = b2 - (a2*b) + (0.25f * a2);
    float d2;

    EllipsePoints(ab, x,y);

    while ((a2 * (y - 0.5f)) > (b2 * (x + 1))) {  /* Region 1 */
        if (d1 < 0)                               /* Select E */
            d1 += b2 * (2*x + 3);
        else {                                    /* Select SE */
            d1 += b2 * (2*x + 3) + a2 * (-2*y + 2);
            y--;
        }
        x++;

        EllipsePoints(ab, x,y);
    }

    d2 = b2*((x + 0.5f)*(x + 0.5f)) + b2*((y - 1)*(y - 1)) - a2*b2;
    while (y > 0) {                               /* Region 2 */
        if (d2 < 0) {                             /* Select SE */
            d2 += b2*(2*x + 2) + a2*(-2*y + 3);
            x++;
        } else
            d2 += a2*(-2*y + 3);                  /* Select S */
        y--;

        EllipsePoints(ab, x,y);
    }
}
#endif

void
oz_draw_arc(const ozWindow *window, const ozGC *gc, int x, int y, int width, int height, float start, float end)
{
#ifdef _WIN32_WCE
    struct ArcBuf ab = {
        window, gc, 0, x, y,
        start, tanf(start * M_PI/180.0f),
        end,   tanf(end   * M_PI/180.0f)
    };

    /* build the points */
    MidpointEllipse(&ab, width, height);
#else
	/* Arc() */
#endif
}
