/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Double buffering code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "oz.h"
#include "ozprivate.h"

typedef struct {
    HDC winDC;
    HDC memDC;
    HBITMAP memBitmap;
    HBITMAP oldBitmap;
}  ozDBPrivate;

/* XXX I don't like calling malloc() here.  Figure out some
   way to do this that keeps oz mostly platform independant */

/* XXX we could probably remove memDC from the struct and just rely
   on what is in the GC when stop is called... */

void
oz_double_buffer_start(ozDoubleBuffer *db, ozGC *gc, int width, int height)
{
    ozDBPrivate *dbp = malloc(sizeof(ozDBPrivate));
    db->data = dbp;

    dbp->winDC = gc->dc;
    dbp->memDC = CreateCompatibleDC(dbp->winDC);
    dbp->memBitmap = CreateCompatibleBitmap(dbp->winDC, width, height);
    dbp->oldBitmap = SelectObject(dbp->memDC, dbp->memBitmap);

    /* set the GC's DC to the new memory DC */
    gc->dc = dbp->memDC;
}

void
oz_double_buffer_stop(ozDoubleBuffer *db, ozGC *gc, int x, int y, int width, int height)
{
    ozDBPrivate *dbp = (ozDBPrivate *)db->data;
    BitBlt(dbp->winDC, x, y, width, height, dbp->memDC, 0, 0, SRCCOPY);

    /* restore the GC's DC to the old DC */
    gc->dc = dbp->winDC;

    /* Deselect and delete new objects */
    DeleteObject(dbp->memBitmap);
    SelectObject(dbp->memDC, dbp->oldBitmap);
    DeleteDC(dbp->memDC);

    free(dbp);
    db->data = NULL;
}
