/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Main oz header
 */

#ifndef oz_h__
#define oz_h__

#ifdef __cplusplus
extern "C" {
#endif

#include "oztypes.h"

/* system functions */
int oz_screen_get_width  (void);

int oz_screen_get_height (void);

int oz_screen_get_depth  (void);

int oz_screen_get_dpi    (void);


/* windowing functions */
ozWindow *oz_window_create     (ozWindow *parent,
                                int x,
                                int y,
                                int width,
                                int height);

void      oz_window_destroy    (ozWindow *win);

ozWindow *oz_window_get_parent (ozWindow *win);

void      oz_window_move       (ozWindow *win,
                                int x,
                                int y);

void      oz_window_resize     (ozWindow *win,
                                int width,
                                int height);

void      oz_window_show       (ozWindow *win);

void      oz_window_hide       (ozWindow *win);

void      oz_window_set_title  (ozWindow *win,
                                const wchar_t *title);

void      oz_window_redraw     (ozWindow *win);

void      oz_window_scroll_set_range (ozWindow *win,
                                      int xmin,
                                      int xmax,
                                      int ymin,
                                      int ymax);

void      oz_window_scroll_set_pos (ozWindow *win,
                                    int x,
                                    int y);

void      oz_window_scroll_get_pos (const ozWindow *win,
                                    uint16 *x,
                                    uint16 *y);

void      oz_window_set_owner  (ozWindow *win,
                                void *owner);

void     *oz_window_get_owner  (ozWindow *win);

#ifdef _WIN32
ozWindow *oz_window_from_hwnd  (HWND hwnd);
#endif

/* drawing functions */
void oz_draw_point      (const ozWindow *window,
                         const ozGC *gc,
                         int x,
                         int y);

void oz_draw_points     (const ozWindow *window,
                         const ozGC *gc,
                         const ozPoint points[],
                         int npoints);

void oz_draw_line       (const ozWindow *window,
                         const ozGC *gc,
                         int x1,
                         int y1,
                         int x2,
                         int y2);

void oz_draw_lines      (const ozWindow *window,
                         const ozGC *gc,
                         const ozPoint points[],
                         int npoints);

void oz_draw_rect       (const ozWindow *window,
                         const ozGC *gc,
                         int x,
                         int y,
                         int width,
                         int height,
                         int fill);

void oz_draw_arc        (const ozWindow *window,
                         const ozGC *gc,
                         int x1,
                         int y1,
                         int x2,
                         int y2,
                         float start,
                         float end);

void oz_draw_polygon    (const ozWindow *window,
                         const ozGC *gc,
                         const ozPoint points[],
                         int npoints,
                         int fill);

void oz_draw_trianglef  (const ozWindow *window,
                         const ozGC *gc,
                         const ozTrianglef t);

void oz_draw_string     (const ozWindow *window,
                         const ozGC *gc,
                         int x,
                         int y,
                         const char *text,
                         int len);

void oz_draw_image      (const ozWindow *window,
                         const ozGC *gc,
                         int x,
                         int y,
                         int xoffset,
                         int yoffset,
                         int width,
                         int height,
                         int dwidth,
                         int dheight,
                         uint8 istile,
                         uint8 bpp,
                         uint32 nbytes,
                         const uint8 *bytes);


/* double buffering */
void oz_double_buffer_start (ozDoubleBuffer *db,
                             ozGC *gc,
                             int width,
                             int height);

void oz_double_buffer_stop  (ozDoubleBuffer *db,
                             ozGC *gc,
                             int x,
                             int y,
                             int width,
                             int height);


/* event functions */
void oz_events_init     (void);

void oz_events_shutdown (void); /* call at exit */

int  oz_events_get      (ozEvent *event);

/* utility functions */
/* XXX these should be moved in to a platform specific utils header */
void  oz_set_hwnd_userdata(HWND window, void *data);
void *oz_get_hwnd_userdata(HWND window);

#ifdef __cplusplus
}
#endif

#endif /* oz_h__ */
