/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Type definitions
 */

#ifndef oztypes_h__
#define oztypes_h__

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>

/* XXX this should be defined if we're not building the client */
#ifdef MOZILLA_CLIENT
#define OZ_EXPORT __declspec(dllexport)
#else
#define OZ_EXPORT
#endif

//typedef char int8;
typedef unsigned char  uint8;

typedef short int16;
typedef unsigned short uint16;

typedef long int32;
typedef unsigned long  uint32;

/* core structures */
typedef struct {
    short x;
    short y;
} ozPoint;

typedef struct {
    float x;
    float y;
} ozPointf;

typedef ozPointf ozTrianglef[3];

typedef struct {
    short x;
    short y;
    short width;
    short height;
} ozRect;

typedef struct {
    uint8 id;
} ozFont;

typedef struct {
    HDC dc;
    unsigned long rgbaq;
    ozFont font;
} ozGC;

typedef struct ozWindow {
    struct ozWindow *parent;
    short x;
    short y;
    short width;
    short height;
    HWND hwnd;
    void *owner;
    int hidden : 1;
} ozWindow;

typedef struct {
    void *data; /* private data */
} ozDoubleBuffer;

/* event structures */
typedef enum {
    OZ_EVENT_NONE       = -1,
    OZ_EVENT_VSYNC      = 0,
    OZ_EVENT_KEY        = 1,
    OZ_EVENT_PAD        = 2,
    OZ_EVENT_MOUSE      = 3,
    OZ_EVENT_EXIT       = 10,
} ozEventType;

typedef struct {
    ozEventType type;
    int field; /* 0 is even field, 1 is odd */
} ozEventVSync;

typedef struct {
    ozEventType type;
    uint16 state; /* Modifier key state */
    uint8 keyval; /* Value of key pressed */
} ozEventKey;

typedef struct {
    ozEventType type;
} ozEventPad;

typedef struct {
    ozEventType type;
    uint8 button; /* Modifier key state */
    ozPoint pt;
} ozEventMouse;

typedef struct {
    ozEventType type;
} ozEventExit;

typedef union {
    ozEventType  type;
    ozEventVSync vsync;
    ozEventKey   key;
    ozEventPad   pad;
    ozEventMouse mouse;
    ozEventExit  exit;
} ozEvent;

#endif /* oztypes_h__ */
