/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Polygon drawing code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "oz.h"
#include "ozprivate.h"

void
oz_draw_polygon(const ozWindow *window, const ozGC *gc, const ozPoint points[], int npoints, int fill)
{
    if (fill) {
        int i;
        int npts = npoints - 1; /* Polygon() automatically closes the polygon */
        POINT stackpts[20];
        POINT* pts = stackpts;
        HBRUSH brush, oldBrush;
        HPEN pen, oldPen;

        if (npts > 20)
            pts = malloc(sizeof(POINT) * npts);

        for (i = 0; i < npts; ++i) {
            pts[i].x = points[i].x;
            pts[i].y = points[i].y;
        }

        pen = CreatePen(PS_NULL, 0, 0);
        oldPen = SelectObject(gc->dc, pen);
        brush = CreateSolidBrush(gc->rgbaq);
        oldBrush = SelectObject(gc->dc, brush);

        Polygon(gc->dc, pts, npts);

        SelectObject(gc->dc, oldBrush);
        DeleteObject(brush);
        SelectObject(gc->dc, oldPen);
        DeleteObject(pen);


        if (pts != stackpts)
            free(pts);

        return;
    }

    /* same as draw_lines(), except 1 less function call */
    _oz_draw_multi(window, gc, OZ_PRIM_LINESTRIP, points, npoints);
}
