/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Rectangle drawing code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "oz.h"
#include "ozprivate.h"

void
oz_draw_rect(const ozWindow *window, const ozGC *gc, int x1, int y1, int width, int height, int fill)
{
    HBRUSH brush;
    RECT nr = { x1, y1, x1 + width, y1 + height };

    if (fill) {
        brush = CreateSolidBrush(gc->rgbaq);
        FillRect(gc->dc, &nr, brush);
        DeleteObject(brush);
    } else {
#ifdef _WIN32_WCE
        POINT pts[5] = {
            { nr.left, nr.top },
            { nr.right, nr.top },
            { nr.right, nr.bottom },
            { nr.left, nr.bottom },
            { nr.left, nr.top }
        };
		HPEN oldPen, pen = CreatePen(PS_SOLID, 1, gc->rgbaq);
        
		oldPen = SelectObject(gc->dc, pen);

        Polyline(gc->dc, pts, 5);
        SelectObject(gc->dc, oldPen);
        DeleteObject(pen);
#else
        brush = CreateSolidBrush(gc->rgbaq);
        FrameRect(gc->dc, &nr, brush);
        DeleteObject(brush);
#endif
    }
}
