/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Text drawing code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "oz.h"
#include "ozprivate.h"

#include "utf8.h"

#define FONT_NAME L"Tahoma"
#define FONT_SIZE 10
/*
#define FONT_NAME L"Verdana"
#define FONT_SIZE 12
*/

#ifndef _WIN32_WCE
/* for debugging only */
static void
compute_text_width(HDC dc, const wchar_t *text, int len)
{
    int i;
    int width = 0;

    SIZE size;
    /* XXX this should take care of kerning, but doesn't seem to */
    GetTextExtentPoint32W(dc, text, len, &size);

    /* XXX we should make this take kerning in to account */
    for (i = 0; i < len; ++i) {
        ABC abc[1];
        GetCharABCWidthsW(dc, text[i], text[i], abc);
        width += abc->abcA + abc->abcB + abc->abcC;
    }
}
#endif

/* XXX this needs to be deleted when the app exits */
static HFONT
get_font()
{
    static HFONT font = NULL;

    if (!font) {
        LOGFONT logFont;
        memset(&logFont, 0, sizeof(logFont));
        logFont.lfHeight = -FONT_SIZE;
        logFont.lfWeight = FW_NORMAL;
#ifndef _WIN32_WCE
        logFont.lfOutPrecision = OUT_TT_ONLY_PRECIS;
#endif
        logFont.lfQuality = NONANTIALIASED_QUALITY;
        lstrcpy(logFont.lfFaceName, FONT_NAME);
        font = CreateFontIndirect(&logFont);
    }

    return font;
}

void
oz_draw_string(const ozWindow *window, const ozGC *gc, int x, int y, const char *text, int len)
{
    HFONT oldFont;

    /* convert from UTF-8 to USC2 */
    int wlen = sizeof(wchar_t) * len;
    wchar_t *wtext = malloc(wlen);
    utf8_to_ucs2(text, &len, wtext, &wlen);

    /* select font in */
    oldFont = SelectObject(gc->dc, get_font());

    /* set the background mode to transparent */
    SetBkMode(gc->dc, TRANSPARENT);

    /* set the text color */
    SetTextColor(gc->dc, gc->rgbaq);

    /* Draw the text */
    ExtTextOutW(gc->dc, x, y, 0, NULL, wtext, wlen, NULL);

    /* Debugging code */
    /* compute_text_width(gc->dc, wtext, wlen); */

    /* restore the old font */
    SelectObject(gc->dc, oldFont);

    /* free the UCS2 text */
    free(wtext);
}
