/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client/server code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2002-2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Brendan Eich <brendan@mozilla.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

/*
 * Anya Byte Codes
 */

#ifndef abc_h__
#define abc_h__

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Anya data types:
 *
 *      byte            unsigned 8 bits
 *      char            UTF-8 character
 *      int16
 *      coord/uint16
 *      int32
 *      uint16          angle is fixed point radians?
 *
 *
 * Anya grammar:
 *
 *      device          ::= display dpi xres yres uid
 *
 *      request         ::= version type device n {char}*
 *
 *
 *      document        ::= header {element}*
 *
 *      header          ::= version w h display-list-size
 *
 *      element         ::= idmap (XXX there can be only one.. move this to header, or the end of the document?)
 *                        | display-element
 *                        | html-element
 *
 *      idmap           ::= dictionary compresseddata
 *
 *      id              ::= uint32
 *
 *      input-element   ::= formid nameid valueid
 *
 *      display-element ::= POINT x y
 *                        | LINE x1 y1 x2 y2
 *                        | RECT x y w h filled
 *                        | ARC x1 y1 x2 y2 start-angle end-angle
 *                        | POINTS n {x y}*
 *                        | POLYLINE n {x y}*
 *                        | POLYGON n {x y}* filled
 *                        | TEXT x y id
 *                        | FONT n
 *                        | COLOR n
 *                        | IMAGE x y w h bpp n {byte}*
 *                        | EVENT n {x y w h}* id
 *
 * Form controls
 *                        | INPUT_FORM input-element method
 *                        | INPUT_HIDDEN input-element
 *                        | INPUT_TEXT input-element x y w h
 *                        | INPUT_SUBMIT input-element x y w h

 * Everything below has yet to be implemented so there may be issues with it
 *
 *                        | BUTTON id x y w h n {display-element}*
 *                        | SUBMIT id x y w h n {char}*
 *                        | CHECKBOX id checked
 *                        | RADIO id checked
 *                        | INPUT id x y w h n {char}*
 *                        | TEXTAREA id x y w h n {char}*
 *                        | SELECT1 id x y w h n {char}*
 *                        | SELECTN id x y w h n {n {char}*}*
 *
 *      html-element    ::= html-tag html-attrs n {html-element}*
 *
 *      html-tag        ::= A | ABBR | ACRONYM | ADDRESS | APPLET | AREA
 *                        | B | BASE | BDO | BGSOUND | BIG | BLINK
 *                        | BLOCKQUOTE | BODY | BR | BUTTON
 *                        | CAPTION | CENTER | CITE | CODE | COL
 *                        | COLGROUP
 *                        | DD | DEL | DFN | DIV | DL | DT
 *                        | EM | EMBED | ENDNOTE |
 *                        | FIELDSET | FONT | FORM | FRAME | FRAMESET
 *                        | H1 | H2 | H3 | H4 | H5 | H6 | HEAD | HR | HTML
 *                        | I | IFRAME | IMG | INPUT | INS | ISINDEX
 *                        | KBD
 *                        | LABEL | LEGEND | LI | LINK | LISTING
 *                        | MAP | MULTICOL
 *                        | OBJECT | OL | OPTGROUP | OPTION
 *                        | P | PARAM | PLAINTEXT | PRE
 *                        | Q
 *                        | RADIO
 *                        | S | SAMP | SCRIPT | SELECT | SMALL | SPACER | SPAN
 *                        | STRIKE | STRONG | STYLE | SUB | SUBMIT | SUP
 *                        | TABLE | TBODY | TD | TEXT | TEXTAREA | TFOOT
 *                        | THEAD | TITLE | TR | TT
 *                        | U | UL
 *                        | VAR
 *
 *      html-attrs      ::= n {id html-attr-value}*
 *
 *      html-attr-value ::= eval(id << 1)
 *                        | eval((n << 1) | 1) {offset length}*
 */

/* anya bytecode list */
enum {
    ABC_EOF      = 0,
    ABC_IDMAP    = 1,

    ABC_NOP      = 4,
    ABC_ERROR    = 5,

    ABC_POINT    = 10,
    ABC_LINE     = 11,
    ABC_RECT     = 12,
    ABC_ARC      = 13,
    ABC_POINTS   = 14,
    ABC_POLYLINE = 15,
    ABC_POLYGON  = 16,
    ABC_TEXT     = 17,
    ABC_FONT     = 18,
    ABC_COLOR    = 19,
    ABC_IMAGE    = 20,

    ABC_EVENT    = 25,

    ABC_HTMLELEMENT    = 100, /* don't use */
    ABC_INPUT_FORM     = 101,
    ABC_INPUT_HIDDEN   = 102,
    ABC_INPUT_TEXT     = 103,
    ABC_INPUT_PASSWORD = 104,
    ABC_INPUT_TEXTAREA = 105,
    ABC_INPUT_SUBMIT   = 106,
    ABC_INPUT_SELECT   = 107,
    ABC_INPUT_CHECKBOX = 108,
    ABC_INPUT_RADIO    = 109
};

#if 0
/* XXX Think about building a table here
 */
enum {
    AOP_NOP    = 0x0
    AOP_BYTE   = 0x1,
    AOP_UINT16 = 0x2,
    AOP_UINT32 = 0x3,
    AOP_ANGLE  = 0x4,
    AOP_XY     = 0x5,
    AOP_ID     = 0x6,
};

#define AOPS(a, b, c, d, e, f, g, h) \
    ((a) | ((b) << 4) | ((c) << 8) | ((d) << 12) | ((e) << 16) | ((f)) << 20 | ((g) << 24) | ((h) << 28))


/* legend: op         val name          image       len use def prec  format */
OPDEF(ABC_NOP,        0,  "nop",        NULL,         1,  0,  0,  0,  JOF_BYTE)
OPDEF(ABC_POINT,     10,  "point",      NULL,         5,  0,  0,  0,  AOPS(AOP_BYTE, AOP_XY))
OPDEF(ABC_LINE,      11,  "line",       NULL,         9,  0,  0,  0,  AOPS(AOP_BYTE, AOP_XY, AOP_XY))
OPDEF(ABC_RECT,      12,  "rect",       NULL,        10,  0,  0,  0,  AOPS(AOP_BYTE, AOP_XY, AOP_XY, AOP_BYTE))
OPDEF(ABC_ARC,       13,  "arc",        NULL,         9,  0,  0,  0,  AOPS(AOP_BYTE, AOP_XY, AOP_XY, AOP_ANGLE, AOP_ANGLE))
OPDEF(ABC_POINTS,    14,  "points",     NULL,         1,  0,  0,  0,  JOF_BYTE)
OPDEF(ABC_POLYLINE , 15,  "polyline",   NULL,         1,  0,  0,  0,  JOF_BYTE)
OPDEF(ABC_POLYGON,   16,  "polygon",    NULL,         1,  0,  0,  0,  JOF_BYTE)
OPDEF(ABC_TEXT,      17,  "text",       NULL,         9,  0,  0,  0,  AOPS(AOP_BYTE, AOP_XY, AOP_ID))
OPDEF(ABC_FONT,      18,  "font",       NULL,         5,  0,  0,  0,  AOPS(AOP_BYTE, AOP_UINT32))
OPDEF(ABC_COLOR,     19,  "color",      NULL,         5,  0,  0,  0,  AOPS(AOP_BYTE, AOP_UINT32))
OPDEF(ABC_IMAGE,     20,  "image",      NULL,        14,  0,  0,  0,  AOPS(AOP_BYTE, AOP_XY, AOP_XY, AOP_BYTE, AOP_ID))
OPDEF(ABC_EVENT,     25,  "event",      NULL,         1,  0,  0,  0,  JOF_BYTE)
#endif

#ifdef __cplusplus
}
#endif

#endif /* abc_h__ */
