/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client/server code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Joe Hewitt <hewitt@netscape.com>
 *    Ben Googer <ben@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#ifndef prims_h__
#define prims_h__

#ifdef __cplusplus
extern "C" {
#endif

#include "oztypes.h"
#include "xcoder.h"

enum {
    ANYA_REQUEST_GET     = 1 << 0,
    ANYA_REQUEST_POST    = 1 << 1
};

enum {
    ANYA_DEVICE_BW       = 1 << 0,
    ANYA_DEVICE_GREY16   = 1 << 1,
    ANYA_DEVICE_COLOR16  = 1 << 2,
    ANYA_DEVICE_COLOR256 = 1 << 3,
    ANYA_DEVICE_COLOR16b = 1 << 4,
    ANYA_DEVICE_COLOR24b = 1 << 5,
    ANYA_DEVICE_COLOR32b = 1 << 6
};

struct Request {
    uint8 version;             /* protocol version */
    uint8 request;             /* request type. GET or POST */
    struct Device {
        uint8  display;
        uint8  dpi;
        uint16 xres;
        uint16 yres;
        uint32 uid;
    } device;
    char *uri;
};


struct Header {
    uint8  version;            /* protocol version.  version == 0 indicates an error */
    uint16 w;                  /* width of the page */
    uint16 h;                  /* height of the page */
    uint32 display_list_size;  /* size of all the structs in the display list */
    uint32 form_list_length;   /* how many form elements are there, including the form itself */
                               /* XXX this may become html element length at some later point */
};

typedef struct HtmlElement {
    uint8 type;
    uint8 tp;

    uint32 nattrs;
    struct _attr {
        uint32 id;
        union {
            uint32 value;
            struct {
                uint32 nvalues;
                struct _attr_value {
                    uint32 offset;
                    uint32 length;
                } *values;
            };
        };
    } *attrs;

    uint32 nkids;
    struct HtmlElement *kids;
} HtmlElement;

/* display elements */
struct Font {
    uint8 type;
    uint8 id;
    uint8 pad[2];
};

struct Color {
    uint8 type;
    uint8 r;
    uint8 b;
    uint8 g;
};

struct Point {
    uint8 type;
    uint8 pad[3];
    uint16 x, y;
};

struct Line {
    uint8 type;
    uint8 pad[3];
    uint16 x1, y1, x2, y2;
};

struct Rect {
    uint8 type;
    uint8 filled;
    uint8 pad[2];
    uint16 x, y, w, h;
};

struct Arc {
    uint8 type;
    uint8 pad[3];
    uint16 x1, y1, x2, y2;
    uint16 start, end;
};

struct Points {
    uint8 type;
    uint8 pad;
    uint16 npoints;
    ozPoint *points;
};

struct Polygon {
    uint8 type;
    uint8 filled;
    uint16 npoints;
    ozPoint *points;
};

struct Polyline {
    uint8 type;
    uint8 pad;
    uint16 npoints;
    ozPoint *points;
};

struct Text {
    uint8 type;
    uint8 pad[3];
    uint16 x, y;
    uint32 id;                 /* text string (IdMap ID) */
};

struct Image {
    uint8 type;
    uint8 bpp;
    uint8 istile;
    uint8 pad;
    uint16 x, y, w, h;
    uint16 destw, desth;
    uint16 xoffset, yoffset;
    uint32 nbytes;
    uint8 *bytes;
};

struct Event {
    uint8 type;
    uint8 pad;
    uint16 nrects;             /* number of rects */
    ozRect *rects;             /* rects */
    uint32 id;                 /* URI (IdMap ID) */
};

struct Input {
    uint8 type;
    uint8 formid;              /* unique form ID */
};

struct InputForm {
    struct Input i; /* XXX we don't really need name or value here */
    uint8 method;              /* 0 = GET.  1 = POST */
    uint8 pad;
    uint32 action;             /* value attribute (IdMap ID) */
};

struct InputHidden {
    struct Input i;
    uint8 pad[2];
    uint32 name;               /* name attribute (IdMap ID) */
    uint32 value;              /* value attribute (IdMap ID) */
};

/* XXX if we add a flag here we could probably move value and data
       in to a union */
struct InputText {
    struct Input i;
    uint8 pad[2];
    uint16 x, y, w, h;
    uint32 name;               /* name attribute (IdMap ID) */
    uint32 value;              /* value attribute (IdMap ID) */
    uint32 maxlength;
    void *data;
};

struct InputCheckbox {
    struct Input i;
    uint8 checked;
    uint8 pad;
    uint16 x, y, w, h;
    uint32 name;               /* name attribute (IdMap ID) */
    void *data;
};

struct InputRadio {
    struct Input i;
    uint8 checked;
    uint8 pad;
    uint16 x, y, w, h;
    uint32 name;               /* name attribute (IdMap ID) */
    uint32 value;              /* value attribute (IdMap ID) */
    void *data;
};

struct InputSubmit {
    struct Input i;
    uint8 pad[2];
    uint16 x, y, w, h;
    uint32 name;               /* name attribute (IdMap ID) */
    uint32 value;              /* value attribute (IdMap ID) */
};

struct InputOption {
    uint32 label;              /* label attribute (IdMap ID) */
    uint32 value;              /* value attribute (IdMap ID) */
};

struct InputSelect {
    struct Input i;
    uint8 iscombo;
    uint8 pad;
    uint16 x, y, w, h;
    uint32 name;               /* name attribute (IdMap ID) */
    uint32 selectedindex;
    uint32 noptions;
    struct InputOption *options;
    void *data;
};

/* client -> server */
OZ_EXPORT int  xcoder_Request(XCoderStream *stream, struct Request *req);

/* server -> client */
OZ_EXPORT int  xcoder_Header(XCoderStream *stream, struct Header *head);

/* idmap */
OZ_EXPORT int  xcoder_IdMap(XCoderStream *stream, struct IdMap *map);

/* 2d prims */
OZ_EXPORT int  xcoder_Point(XCoderStream *stream, struct Point *p);
OZ_EXPORT int  xcoder_Line(XCoderStream *stream, struct Line *l);
OZ_EXPORT int  xcoder_Rect(XCoderStream *stream, struct Rect *r);
OZ_EXPORT int  xcoder_Arc(XCoderStream *stream, struct Arc *a);
OZ_EXPORT int  xcoder_Points(XCoderStream *stream, struct Points *p);
OZ_EXPORT int  xcoder_Polyline(XCoderStream *stream, struct Polyline *p);
OZ_EXPORT int  xcoder_Polygon(XCoderStream *stream, struct Polygon *p);
OZ_EXPORT int  xcoder_Text(XCoderStream *stream, struct Text *t);
OZ_EXPORT int  xcoder_Image(XCoderStream *stream, struct Image *i);
OZ_EXPORT int  xcoder_Font(XCoderStream *stream, struct Font *f);
OZ_EXPORT int  xcoder_Color(XCoderStream *stream, struct Color *c);

OZ_EXPORT int  xcoder_Event(XCoderStream *stream, struct Event *e);

OZ_EXPORT int  xcoder_HtmlElement(XCoderStream *stream, HtmlElement *element);


/* forms */
OZ_EXPORT int  xcoder_InputForm(XCoderStream *stream, struct InputForm *i);
OZ_EXPORT int  xcoder_InputHidden(XCoderStream *stream, struct InputHidden *i);
OZ_EXPORT int  xcoder_InputText(XCoderStream *stream, uint8 abc, struct InputText *i);
OZ_EXPORT int  xcoder_InputSubmit(XCoderStream *stream, struct InputSubmit *i);
OZ_EXPORT int  xcoder_InputSelect(XCoderStream *stream, struct InputSelect *i);
OZ_EXPORT int  xcoder_InputCheckbox(XCoderStream *stream, struct InputCheckbox *i);
OZ_EXPORT int  xcoder_InputRadio(XCoderStream *stream, struct InputRadio *i);

#ifdef __cplusplus
}
#endif

#endif /* prims_h__ */
