/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya client/server code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Brendan Eich <brendan@mozilla.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#ifndef xcoder_h__
#define xcoder_h__

#ifdef PALMOS_SDK_VERSION
#define ANYA_NO_COMPRESSED_STREAM 1
#define XCODER_NO_WRITEFILE 1
#endif

#ifdef __cplusplus
extern "C" {
#endif
#include <stdlib.h>
#include <stdio.h>

#include "oztypes.h"

#ifdef ANYA_NO_COMPRESSED_STREAM
#undef ANYA_COMPRESSED_STREAM
#define ANYA_NO_COMPRESSED_STREAM_READ
#define ANYA_NO_COMPRESSED_STREAM_WRITE
#else
#define ANYA_COMPRESSED_STREAM
#endif

#ifdef ANYA_COMPRESSED_STREAM
#include "zlib.h"
#endif

enum XCoderMode {
    XCODER_READ      = 1 << 0,
    XCODER_WRITE     = 1 << 1,
    XCODER_WRITEFILE = 1 << 2,

    XCODER_COMPRESS  = 1 << 5
};

#define XCODER_BUFFER_SIZE      512

typedef struct XCoderStream {
    int         fd;
    uint8       flags;
    int         count;
    int         cursor;
    char        buffer[XCODER_BUFFER_SIZE];
#ifdef ANYA_COMPRESSED_STREAM
    z_stream    *zstream;
    char        cbuffer[XCODER_BUFFER_SIZE];
#endif
} XCoderStream;

#define FLAG_READ(s)      (s->flags & XCODER_READ)
#define FLAG_WRITE(s)     (s->flags & XCODER_WRITE)
#define FLAG_WRITEFILE(s) (s->flags & XCODER_WRITEFILE)
#define FLAG_COMPRESS(s)  (s->flags & XCODER_COMPRESS)

OZ_EXPORT int  xcoder_Open(XCoderStream *stream, int fd, int flags);
OZ_EXPORT void xcoder_Close(XCoderStream *stream);
OZ_EXPORT int  xcoder_FillBuffer(XCoderStream *stream);
OZ_EXPORT int  xcoder_FlushBuffer(XCoderStream *stream);
OZ_EXPORT int  xcoder_FlushBufferEnd(XCoderStream *stream); /* probably move to Close() */
OZ_EXPORT int  xcoder_CanAccess(XCoderStream *stream, int nbytes);
OZ_EXPORT int  xcoder_Read8(XCoderStream *stream, uint8 *ip);
OZ_EXPORT int  xcoder_Read16(XCoderStream *stream, uint16 *ip);
OZ_EXPORT int  xcoder_Read32(XCoderStream *stream, uint32 *ip);
OZ_EXPORT int  xcoder_ReadBytes(XCoderStream *stream, uint32 *len, uint8 **sp);
OZ_EXPORT int  xcoder_ReadString(XCoderStream *stream, char **sp);
OZ_EXPORT int  xcoder_ReadNString(XCoderStream *stream, uint32 *len, char **sp);


#ifdef __cplusplus
}
#endif

#endif /* xcoder_h__ */
