/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya server code
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#include "anyaDataTransferManager.h"

#include "prims.h" 

#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

anyaDataTransferManager::anyaDataTransferManager()
{
    Init();
}

anyaDataTransferManager::~anyaDataTransferManager()
{
    Destroy();
}


/* XXX need a better scheme for filenames than using a single filename since with
   multiple processes you're going to be either hitting file locks or writing over
   the same file... */
void
anyaDataTransferManager::Init()
{
    mInited = PR_TRUE;
    mLen = 0;
    mFormLen = 0;
    mFD = _open("c:\\anya.data", _O_WRONLY | _O_CREAT | _O_TRUNC | _O_BINARY, _S_IWRITE);
    xcoder_Open(&mStream, mFD, XCODER_WRITEFILE);
    idmap_Init(&mIdMap);
}

void
anyaDataTransferManager::Destroy()
{
    if (!mInited)
        return;

    idmap_Destroy(&mIdMap);
    xcoder_Close(&mStream);
    _close(mFD);
    mInited = PR_FALSE;
}

NS_IMPL_ISUPPORTS1(anyaDataTransferManager, anyaIDataTransferManager)

XCoderStream* 
anyaDataTransferManager::GetXCoder()
{
    if (!mInited)
        Init();
    return &mStream;
}

struct IdMap* 
anyaDataTransferManager::GetIdMap()
{
    if (!mInited)
        Init();
    return &mIdMap;
}

void 
anyaDataTransferManager::Inc(unsigned long n) 
{
    if (!mInited)
        Init();
    mLen += n;
}

void 
anyaDataTransferManager::IncFormCount(unsigned long n) 
{
    if (!mInited)
        Init();
    mFormLen += n;
}

void
anyaDataTransferManager::Flush()
{
    if (!mInited)
        Init();

    idmap_Compress(&mIdMap);
    xcoder_IdMap(&mStream, &mIdMap);
    xcoder_FlushBuffer(&mStream);

    FILE *f;
    f = fopen("c:\\anya_len.txt", "w+");
    fprintf(f, "%d %d", mLen, mFormLen);
    fclose(f);

    Destroy();
}

void
anyaDataTransferManager::SetRequest(struct Request* aRequest)
{
    mRequest = aRequest;
}

struct Request*
anyaDataTransferManager::GetRequest()
{
    return mRequest;
}
