/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya server code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#include "anyaDeviceContext.h"

#include "anyaRenderingContext.h"

#include "nsCOMPtr.h"
#include "nsIView.h"

NS_IMPL_ISUPPORTS_INHERITED0(anyaDeviceContext, DeviceContextImpl)

anyaDeviceContext::anyaDeviceContext()
{
    NS_INIT_ISUPPORTS();

    mDevUnitsToAppUnits = 1.0f;
    mAppUnitsToDevUnits = 1.0f;
    mCPixelScale = 1.0f;
    mZoom = 1.0f;
    mTextZoom = 1.0f;
}

anyaDeviceContext::~anyaDeviceContext()
{
}

NS_IMETHODIMP anyaDeviceContext::Init(nsNativeWidget aWidget)
{
    mTwipsToPixels = 96 / (float)NSIntPointsToTwips(72);
    mPixelsToTwips = 1.0f / mTwipsToPixels;

    //DeviceContextImpl::CommonInit();

    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::CreateRenderingContext(nsIView *aView,
                                                        nsIRenderingContext *&aContext)
{
    nsCOMPtr<nsIWidget> widget;
    aView->GetWidget(*getter_AddRefs(widget));

    return CreateRenderingContext(widget, aContext);
}

NS_IMETHODIMP anyaDeviceContext::CreateRenderingContext(nsDrawingSurface aSurface,
                                                        nsIRenderingContext *&aContext)
{
    nsresult rv;

    aContext = nsnull;
    nsCOMPtr<nsIRenderingContext> pContext;
    rv = CreateRenderingContextInstance(*getter_AddRefs(pContext));
    if (NS_SUCCEEDED(rv)) {
        rv = pContext->Init(this, aSurface);
        if (NS_SUCCEEDED(rv)) {
            aContext = pContext;
            NS_ADDREF(aContext);
        }
    }

    return rv;
}

NS_IMETHODIMP anyaDeviceContext::CreateRenderingContext(nsIWidget *aWidget,
                                                         nsIRenderingContext *&aContext)
{
    nsresult rv;

    aContext = nsnull;
    nsCOMPtr<nsIRenderingContext> pContext;
    rv = CreateRenderingContextInstance(*getter_AddRefs(pContext));
    if (NS_SUCCEEDED(rv)) {
        rv = pContext->Init(this, aWidget);
        if (NS_SUCCEEDED(rv)) {
            aContext = pContext;
            NS_ADDREF(aContext);
        }
    }

    return rv;
}

NS_IMETHODIMP anyaDeviceContext::CreateRenderingContext(nsIRenderingContext *&aContext)
{
    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::CreateRenderingContextInstance(nsIRenderingContext *&aContext)
{
    nsCOMPtr<nsIRenderingContext> renderingContext = new anyaRenderingContext();
    if (!renderingContext)
        return NS_ERROR_OUT_OF_MEMORY;

    aContext = renderingContext;
    NS_ADDREF(aContext);

    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::SupportsNativeWidgets(PRBool &aSupportsWidgets)
{
    aSupportsWidgets = PR_TRUE;
    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::GetScrollBarDimensions(float &aWidth, float &aHeight) const
{
    aWidth = 10.0f * mPixelsToTwips;
    aHeight = 10.0f * mPixelsToTwips;
    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::GetSystemFont(nsSystemFontID aID, nsFont *aFont) const
{
    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::GetDrawingSurface(nsIRenderingContext &aContext,
                                                   nsDrawingSurface &aSurface)
{
    //aContext.CreateDrawingSurface(nsnull, 0, aSurface);
    return nsnull == aSurface ? NS_ERROR_OUT_OF_MEMORY : NS_OK;  
}


NS_IMETHODIMP anyaDeviceContext::CheckFontExistence(const nsString& aFaceName)
{
    return NS_OK;
}

NS_IMETHODIMP anyaDeviceContext::GetDepth(PRUint32& aDepth)
{
    aDepth = 24;
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::GetPaletteInfo(nsPaletteInfo& aPaletteInfo)
{
    aPaletteInfo.isPaletteDevice = PR_FALSE;
    aPaletteInfo.sizePalette = 0;
    aPaletteInfo.numReserved = 0;
    aPaletteInfo.palette = nsnull;
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::ConvertPixel(nscolor aColor, PRUint32 & aPixel)
{
    aPixel = aColor;
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::GetDeviceSurfaceDimensions(PRInt32 &aWidth, PRInt32 &aHeight)
{
    aWidth = NSToIntRound(float(1000) * mDevUnitsToAppUnits);
    aHeight = NSToIntRound(float(1000) * mDevUnitsToAppUnits);
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::GetRect(nsRect &aRect)
{
    aRect.x = 0;
    aRect.y = 0;
    this->GetDeviceSurfaceDimensions(aRect.width, aRect.height);

    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::GetClientRect(nsRect &aRect)
{
    return this->GetRect(aRect);
}



/*
 * below methods are for printing and are not implemented
 */
NS_IMETHODIMP anyaDeviceContext::GetDeviceContextFor(nsIDeviceContextSpec *aDevice,
                                                     nsIDeviceContext *&aContext)
{
    /* we don't do printing */
    return NS_ERROR_NOT_IMPLEMENTED;
}


NS_IMETHODIMP anyaDeviceContext::PrepareDocument(PRUnichar * aTitle, 
                                                 PRUnichar*  aPrintToFileName)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::BeginDocument(PRUnichar*  aTitle, 
                                               PRUnichar*  aPrintToFileName,
                                               PRInt32     aStartPage, 
                                               PRInt32     aEndPage)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::EndDocument(void)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::AbortDocument(void)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::BeginPage(void)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::EndPage(void)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::SetAltDevice(nsIDeviceContext* aAltDC)
{
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::GetAltDevice(nsIDeviceContext** aAltDC)
{
    *aAltDC = nsnull;
    return NS_OK;
}


NS_IMETHODIMP anyaDeviceContext::SetUseAltDC(PRUint8 aValue, PRBool aOn)
{
    return NS_OK;
}

