/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya server code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#include "anyaImage.h"

#include "nsIServiceManager.h"
#include "anyaIDataTransferManager.h"

#include "xcoder.h"
#include "prims.h"

anyaImage::anyaImage()
 : mWidth(0),
   mHeight(0)
{
}

anyaImage::~anyaImage()
{
}

NS_IMPL_ISUPPORTS1(anyaImage, nsIImage);

//////////////////////////////////////////////
//// nsIImage

nsresult
anyaImage::Init(PRInt32 aWidth, PRInt32 aHeight, PRInt32 aDepth, nsMaskRequirements aMaskRequirements)
{
    mWidth = aWidth;
    mHeight = aHeight;

    mImage.depth = aDepth;
    mImage.bpr = (aWidth * aDepth) >> 5;
    if (((PRUint32)aWidth * aDepth) & 0x1F)
        mImage.bpr++;
    mImage.bpr <<= 2;
    mImage.size = mImage.bpr * aHeight;
    mImage.bits = (PRUint8*)new PRUint8[mImage.size];

    switch(aMaskRequirements)
    {
    case nsMaskRequirements_kNeeds1Bit:
        mAlphaImage.depth = 1;
        mAlphaImage.bpr = (aWidth + 7) / 8;
        break;
    case nsMaskRequirements_kNeeds8Bit:
        mAlphaImage.depth = 8;
        mAlphaImage.bpr = aWidth;
        break;
    default:
        break;
    }

    if (aMaskRequirements != nsMaskRequirements_kNoMask) {
        // 32-bit align each row
        mAlphaImage.bpr = (mAlphaImage.bpr + 3) & ~0x3;
        mAlphaImage.bits = (PRUint8*)new PRUint8[mAlphaImage.bpr * aHeight];
    }

    return NS_OK;
}

PRInt32
anyaImage::GetBytesPix()
{
    return mImage.depth / 8;
}

PRBool
anyaImage::GetIsRowOrderTopToBottom()
{
    return PR_FALSE;
}

PRInt32
anyaImage::GetWidth()
{
    return mWidth;
}

PRInt32
anyaImage::GetHeight()
{
    return mHeight;
}

PRUint8 *
anyaImage::GetBits()
{
    return mImage.bits;
}

PRInt32
anyaImage::GetLineStride()
{
    return mImage.bpr;
}

PRBool
anyaImage::GetHasAlphaMask()
{
    return (mAlphaImage.depth != 0);
}

PRUint8 *
anyaImage::GetAlphaBits()
{
    return mAlphaImage.bits;
}

PRInt32
anyaImage::GetAlphaLineStride()
{
    return mAlphaImage.bpr;
}

void
anyaImage::ImageUpdated(nsIDeviceContext *aContext, PRUint8 aFlags, nsRect *aUpdateRect)
{
}

nsresult
anyaImage::Optimize(nsIDeviceContext* aContext)
{
    return NS_OK;
}

nsColorMap *
anyaImage::GetColorMap()
{
    return NULL;
}

NS_IMETHODIMP
anyaImage::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface, PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaImage::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface, PRInt32 aSX, PRInt32 aSY, PRInt32 aSWidth, PRInt32 aSHeight,
                PRInt32 aDX, PRInt32 aDY, PRInt32 aDWidth, PRInt32 aDHeight)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaImage::DrawTile(nsIRenderingContext &aContext,
                    nsDrawingSurface aSurface,
                    PRInt32 aSXOffset, PRInt32 aSYOffset,
                    const nsRect &aTileRect)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaImage::DrawToImage(nsIImage* aDstImage, PRInt32 aDX, PRInt32 aDY, PRInt32 aDWidth, PRInt32 aDHeight)
{
    return NS_OK;
}

PRInt8
anyaImage::GetAlphaDepth()
{
    return mAlphaImage.depth;
}

void *
anyaImage::GetBitInfo()
{
    return NULL;
}

NS_IMETHODIMP
anyaImage::LockImagePixels(PRBool aMaskPixels)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaImage::UnlockImagePixels(PRBool aMaskPixels)
{
    return NS_OK;
}
