/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya server code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#include "anyaAppShell.h"

#include <windows.h>

NS_IMPL_ISUPPORTS1(anyaAppShell, nsIAppShell)

static PRBool gRunning = 0;

anyaAppShell::anyaAppShell()
{
    NS_INIT_ISUPPORTS();
}

anyaAppShell::~anyaAppShell()
{
}

NS_IMETHODIMP
anyaAppShell::Create(int *argc, char **argv)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaAppShell::Run()
{
    gRunning = 1;

    // Stay inside native event loop until somebody calls ::Exit()
    MSG msg;
    HANDLE hFakeEvent = ::CreateEvent(NULL, TRUE, FALSE, NULL);
    while (gRunning) {
        while (::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
            if (!::GetMessage(&msg, NULL, 0, 0))
                gRunning = PR_FALSE;

            ::TranslateMessage(&msg);
            ::DispatchMessage(&msg);
        }

        // Do idle stuff
        ::MsgWaitForMultipleObjects(1, &hFakeEvent, FALSE, 100, QS_ALLEVENTS);
    }

    ::CloseHandle(hFakeEvent);

    return NS_OK;
}

NS_IMETHODIMP
anyaAppShell::Spinup()
{
    return NS_OK;
}

NS_IMETHODIMP
anyaAppShell::Spindown()
{
    return NS_OK;
}

NS_IMETHODIMP
anyaAppShell::ListenToEventQueue(nsIEventQueue * aQueue, PRBool aListen)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaAppShell::GetNativeEvent(PRBool & aRealEvent, void * & aEvent)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaAppShell::DispatchNativeEvent(PRBool aRealEvent, void * aEvent)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaAppShell::Exit()
{
    gRunning = 0;
    return NS_OK;
}
