/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya server code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#include "nsIGenericFactory.h"
#include "nsIModule.h"
#include "nsCOMPtr.h"
#include "nsWidgetsCID.h"

#include "anyaAppShell.h"
#include "anyaWindow.h"
#include "anyaToolkit.h"
#include "anyaLookAndFeel.h"
#include "anyaBidiKeyboard.h"

NS_GENERIC_FACTORY_CONSTRUCTOR(anyaAppShell)
NS_GENERIC_FACTORY_CONSTRUCTOR(anyaWindow)
NS_GENERIC_FACTORY_CONSTRUCTOR(anyaToolkit)
NS_GENERIC_FACTORY_CONSTRUCTOR(anyaLookAndFeel)
NS_GENERIC_FACTORY_CONSTRUCTOR(anyaBidiKeyboard)

static const nsModuleComponentInfo components[] =
{
  { "Anya AppShell",
    NS_APPSHELL_CID,
    "@mozilla.org/widget/appshell/anya;1",
    anyaAppShellConstructor },
  { "Anya Toolkit",
    NS_TOOLKIT_CID,
    "@mozilla.org/widget/toolkit/anya;1",
    anyaToolkitConstructor },
  { "Anya nsWindow",
    NS_WINDOW_CID,
    "@mozilla.org/widgets/window/anya;1",
    anyaWindowConstructor },
  { "Anya child nsWindow",
    NS_CHILD_CID,
    "@mozilla.org/widgets/child_window/anya;1",
    anyaWindowConstructor },
  { "Anya Look And Feel",
    NS_LOOKANDFEEL_CID,
    "@mozilla.org/widget/lookandfeel/anya;1",
    anyaLookAndFeelConstructor },
  { "Anya Bidi Keyboard",
    NS_BIDIKEYBOARD_CID,
    "@mozilla.org/widget/bidikeyboard;1",
    anyaBidiKeyboardConstructor },
  
  { "Horiz Scrollbar",
    NS_HORZSCROLLBAR_CID,
    "@mozilla.org/widgets/horizscroll/anya;1",
    anyaWindowConstructor },
  { "Vert Scrollbar",
    NS_VERTSCROLLBAR_CID,
    "@mozilla.org/widgets/vertscroll/anya;1",
    anyaWindowConstructor },
};

PR_STATIC_CALLBACK(void)
anyaWidgetModuleDtor(nsIModule *self)
{
}

NS_IMPL_NSGETMODULE_WITH_DTOR(anyaWidgetModule,
                              components,
                              anyaWidgetModuleDtor)
