/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Anya server code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by the Initial Developer are
 * Copyright (C) 2003 the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Stuart Parmenter <pavlov@netscape.com>
 *    Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

#include "anyaWindow.h"
#include "nsGUIEvent.h"
#include "nsIRenderingContext.h"
#include "nsIComponentManager.h"
#include "nsGFXCIID.h"

#include <windows.h>

static NS_DEFINE_IID(kRenderingContextCID, NS_RENDERING_CONTEXT_CID);

NS_IMPL_ISUPPORTS_INHERITED0(anyaWindow, nsBaseWidget)

anyaWindow::anyaWindow() :
  mEnabled(PR_FALSE),
  mIsVisible(PR_FALSE)
{
    mBounds = nsRect(0, 0, 320, 240);
}

anyaWindow::~anyaWindow()
{
}

//////////////////////////////////////////////
//// nsIWidget

NS_IMETHODIMP
anyaWindow::Create(nsIWidget *aParent,
                   const nsRect     &aRect,
                   EVENT_CALLBACK   aHandleEventFunction,
                   nsIDeviceContext *aContext,
                   nsIAppShell      *aAppShell,
                   nsIToolkit       *aToolkit,
                   nsWidgetInitData *aInitData)
{
    nsIWidget *baseParent = aInitData &&
                 (aInitData->mWindowType == eWindowType_dialog ||
                  aInitData->mWindowType == eWindowType_toplevel) ?
                  nsnull : aParent;

    BaseCreate(baseParent, aRect, aHandleEventFunction, aContext, 
               aAppShell, aToolkit, aInitData);
    
    mBounds = aRect;

    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Create(nsNativeWidget aParent,
                   const nsRect     &aRect,
                   EVENT_CALLBACK   aHandleEventFunction,
                   nsIDeviceContext *aContext,
                   nsIAppShell      *aAppShell,
                   nsIToolkit       *aToolkit,
                   nsWidgetInitData *aInitData)
{
    BaseCreate(nsnull, aRect, aHandleEventFunction, aContext, 
               aAppShell, aToolkit, aInitData);

    mBounds = aRect;

    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Destroy(void)
{
    nsBaseWidget::Destroy();
    mParent = nsnull;
    mEventCallback = nsnull;

    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::SetParent(nsIWidget* aNewParent)
{
    mParent = aNewParent;
    return NS_OK;
}

nsIWidget*
anyaWindow::GetParent(void)
{
    nsIWidget *ret;
    ret = mParent;
    NS_IF_ADDREF(ret);
    return ret;
}

NS_IMETHODIMP
anyaWindow::Show(PRBool aState)
{
    mIsVisible = aState;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::SetModal(PRBool aModal)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::IsVisible(PRBool & aState)
{
    aState = mIsVisible;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::ConstrainPosition(PRBool aAllowSlop,
                              PRInt32 *aX,
                              PRInt32 *aY)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::Move(PRInt32 aX, PRInt32 aY)
{
    mBounds.x = aX;
    mBounds.y = aY;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Resize(PRInt32 aWidth,
                   PRInt32 aHeight,
                   PRBool  aRepaint)
{
    mBounds.width = aWidth;
    mBounds.height = aHeight;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Resize(PRInt32 aX,
                   PRInt32 aY,
                   PRInt32 aWidth,
                   PRInt32 aHeight,
                   PRBool aRepaint)
{
    Move(aX, aY);
    Resize(aWidth, aHeight, aRepaint);
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::PlaceBehind(nsIWidget *aWidget, PRBool aActivate)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::Enable(PRBool aState)
{
    mEnabled = aState;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::IsEnabled(PRBool *aState)
{
    *aState = mEnabled;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::SetFocus(PRBool aRaise)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::GetBounds(nsRect &aRect)
{
    aRect = mBounds;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::GetScreenBounds(nsRect &aRect)
{
    return GetBounds(aRect);
}

NS_IMETHODIMP
anyaWindow::GetClientBounds(nsRect &aRect)
{
    return GetBounds(aRect);
}

NS_IMETHODIMP
anyaWindow::GetBorderSize(PRInt32 &aWidth, PRInt32 &aHeight)
{
    aWidth = aHeight = 0;
    return NS_OK;
}

nsIFontMetrics* anyaWindow::GetFont(void)
{
    return nsnull;
}

NS_IMETHODIMP
anyaWindow::SetFont(const nsFont &aFont)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

nsCursor anyaWindow::GetCursor(void)
{
    return mCursor;
}

NS_IMETHODIMP
anyaWindow::SetCursor(nsCursor aCursor)
{
    mCursor = aCursor;
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Validate()
{
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Invalidate(PRBool aIsSynchronous)
{
    // We ignore invalidate calls, because they may happy multiple times
    // during the page load process.  Currently, the desired policy for
    // Anya is to wait until the document loads, and then do one complete
    // paint for the entire document by calling Update().

    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Invalidate(const nsRect & aRect, PRBool aIsSynchronous)
{
    Invalidate(aIsSynchronous);
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::InvalidateRegion(const nsIRegion* aRegion, PRBool aIsSynchronous)
{
    Invalidate(aIsSynchronous);
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::Update()
{
    return Paint();
}

NS_IMETHODIMP
anyaWindow::SetColorMap(nsColorMap *aColorMap)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::ScrollWidgets(PRInt32 aDx, PRInt32 aDy)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::ScrollRect(nsRect &aSrcRect, PRInt32 aDx, PRInt32 aDy)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

void* anyaWindow::GetNativeData(PRUint32 aDataType)
{
    return nsnull;
}
void anyaWindow::FreeNativeData(void * data, PRUint32 aDataType)
{
}

NS_IMETHODIMP
anyaWindow::SetTitle(const nsString& aTitle)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::SetIcon(const nsAString& anIconSpec)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::SetMenuBar(nsIMenuBar * aMenuBar)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP
anyaWindow::ShowMenuBar(PRBool aShow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::WidgetToScreen(const nsRect& aOldRect, nsRect& aNewRect)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::ScreenToWidget(const nsRect& aOldRect, nsRect& aNewRect)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::BeginResizingChildren(void)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::EndResizingChildren(void)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::GetPreferredSize(PRInt32& aWidth, PRInt32& aHeight)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::SetPreferredSize(PRInt32 aWidth, PRInt32 aHeight)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::DispatchEvent(nsGUIEvent* aEvent, nsEventStatus & aStatus)
{
    return NS_OK;
}

NS_IMETHODIMP
anyaWindow::EnableDragDrop(PRBool aEnable)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

void
anyaWindow::ConvertToDeviceCoordinates(nscoord &aX,nscoord &aY)
{
}

NS_IMETHODIMP
anyaWindow::CaptureMouse(PRBool aCapture)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::GetWindowClass(char *aClass)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::SetWindowClass(char *aClass)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::CaptureRollupEvents(nsIRollupListener * aListener, PRBool aDoCapture, PRBool aConsumeRollupEvent)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::ModalEventFilter(PRBool aRealEvent, void *aEvent, PRBool *aForWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
anyaWindow::GetAttention()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

//////////////////////////////////////////////
//// anyaWindow protected

nsresult
anyaWindow::Paint()
{
    // Create a paint event
    nsPaintEvent event;
    event.time = ::GetMessageTime();
    event.message = NS_PAINT;
    event.eventStructType = NS_PAINT_EVENT;
    nsRect bounds = mBounds;
    event.rect = &bounds;
    event.region = nsnull;
    event.widget = this;
    NS_ADDREF(event.widget);

    event.renderingContext = GetRenderingContext();

    // Dispatch the event to our callback (the view)
    if (mEventCallback)
        mEventCallback(&event);

    NS_RELEASE(event.renderingContext);

    return NS_OK;
}
